#!/usr/bin/perl -w

# Copyright 2014, 2016 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use Math::BigInt try=>'GMP';
use Math::BigRat;
use List::Util 'max','min','sum';
use Math::PlanePath::FractionsTree;
use Math::PlanePath::RationalsTree;
use Math::Geometry::Planar;
use Memoize 'memoize';

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyRationals ':all';

# cf F(3n)   = 3*F(n-1)*F(n)*F(n+1) + 2*F(n)^3
#

# my $s = sum_altF($n);
# my $f = (-1)**$n * F($n-1) - 1;

my @identities
  = (
     
     
     #----------------------------------------------------
     [ "sum i=1 to n of F(3i-1)",
       { anum => "A049651" },
       \&sum_F3iminus1,
       
       sub {
         my ($n) = @_;
         return (F(3*$n+1) - 1)/2;
       },
     ],
     
     #----------------------------------------------------
     [ "sum i=1 to n of F(n)*F(n)*F(n+2)",
       { anum => "grep" },
       sub {
         my ($n) = @_;
         return sum(0, map {F($_)*F($_)*F($_+2)} range($n));
       },
       
       sub {
         my ($n) = @_;
         return (
                 + 2 * F($n)*F($n+1)*F($n+2)
                 + F($n+1)**3
                 + F($n)**3
                 - 3*(-1)**$n * F($n-1)
                 + 2
                ) / 4;
       },
       sub {
         my ($n) = @_;
         return (+ F($n)*F($n+1)*F($n+2)/2
                 + (F(3*$n+2) - 6*(-1)**$n * F($n-1) + 5) / 10);
       },
       sub {
         my ($n) = @_;
         return (+ sum(0, map {F($_)*F($_)*F($_+1)} range($n))
                 + sum(0, map {F($_)*F($_)*F($_)} range($n)));
       },
     ],
     
     #----------------------------------------------------
     [ "F(3n-1), cf cubes",
       { anum => "A015448" },
       sub { my ($n) = @_;
             return F(3*$n-1);
           },
       
       
       sub { my ($n) = @_;
             return (+ 5*F($n)**3
                     + 5*F($n-1)**3
                     + 3*(-1)**$n*F($n-2)
                    ) / 2;
           },
       sub { my ($n) = @_;
             return (F($n+1)**3 + 2*F($n)**3 - F($n-2)**3) / 2;
           },
     ],
     
     [ "F(3n+1), cf cubes",
       { anum => "A033887" },
       sub { my ($n) = @_;
             return F(3*$n+1);
           },
       
       sub { my ($n) = @_;
             return (3*F($n+1)**3 + 4*F($n)**3 - 2*F($n-1)**3 - F($n-2)**3) / 2;
           },
     ],
     
     [ "F(3n), cf cubes",
       { anum => "A014445" },
       sub { my ($n) = @_;
             return F(3*$n);
           },
       
       sub { my ($n) = @_;
             return F($n+1)**3 + F($n)**3 - F($n-1)**3;
           },
     ],
     
     
     #----------------------------------------------------
     [ "F(n)*F(n)*F(n+2)",
       { anum => "grep" },
       sub {
         my ($n) = @_;
         return F($n)*F($n)*F($n+2);
       },
       
       sub {
         my ($n) = @_;
         return (F(3*$n+2) - (-1)**$n * (4*F($n) + F($n+1))) / 5;
       },
       sub {
         my ($n) = @_;
         return (F(3*$n+2) - (-1)**$n * (3*F($n) + F($n+2))) / 5;
       },
       sub {
         my ($n) = @_;
         return (F(3*$n) - 3 * (-1)**$n * F($n) + F(3*$n+1) - (-1)**$n * F($n+2)) / 5;
       },
       sub {
         my ($n) = @_;
         return ((F(3*$n) - 3 * (-1)**$n * F($n)) / 5
                 + (F(3*$n+1) - (-1)**$n * F($n+2)) / 5);
       },
     ],
     
     #----------------------------------------------------
     [ "hull centroid -- sum side triangles",
       { anum => "grep",
         n_start => 3,
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           my $area = F($i)**2 / 2;
           
           my $cp = (0 + 0 + F($i)) / 3;
           my $cq = (F($i+1)    # q axis
                     + F($i+2)  # q axis
                     + F($i+1) + ($d-$i+1)*F($i)  # sheared
                    ) / 3;
           my $c = ($cp + $cq)/2;
           # $c = (3*F($i+2) + ($d-$i)*F($i)) / 6;
           my $ca2 = $c * 2*$area;
           $total += $ca2;
         }
         return $total;
       },
       
       sub {
         my ($d) = @_;
         return (
                 2 * F(3*$d+5)
                 + 3 * F(3*$d+4)
                 - 24 * (-1)**$d * F($d+1)
                 - 30*$d
                 - 235
                ) / 120;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         $total +=   2 * F(3*$d+5);
         $total +=   3 * F(3*$d+4);
         $total += -24 * (-1)**$d * F($d+1);
         $total += - 30*$d;
         $total += -235;
         return $total / 120;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         $total += 6 * F(3*$d+4);
         $total += F(3*$d+1);
         $total += -24 * (-1)**$d * F($d+1);
         $total += - 30*$d;
         $total += -235;
         return $total / 120;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         $total += 6 * F(3*$d+4);
         $total += F(3*$d+1);
         $total += -24 * (-1)**$d * F($d);
         $total += -24 * (-1)**$d * F($d-1);
         $total += - 30*$d;
         $total += -235;
         return $total / 120;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         $total +=  12* (-1)**$d * F($d-3);
         $total += F(3*$d+1);
         $total += - 30*$d;
         $total += -12 * (-1)**$d * F($d);
         $total += -48 * (-1)**$d * F($d-1);
         $total += 6 * F(3*$d+4);
         $total += -199 - 36;
         return $total / 120;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         $total +=   3* (-1)**$d * F($d-3)  + 3*$d - 9;
         $total += (F(3*$d+1) - 42*$d) / 4;
         $total +=  -3 * (-1)**$d * F($d)  ;
         $total += -12 * (-1)**$d * F($d-1);
         $total += F(3*$d+4)*3/2;
         $total += -199/4;
         return $total / 30;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           # $total +=  - 3* ($d-$i) * (-1)**($i)*F($i);
         }
         $total +=  -3* ( - (-1)**$d * F($d-3) - $d + 3);
         $total += (F(3*$d+1) - 42*$d) / 4;
         $total +=  -3 * (-1)**$d * F($d)  ;
         $total += -12 * (-1)**$d * F($d-1);
         $total += F(3*$d+4)*3/2;
         $total += -199/4;
         return $total / 30;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           $total +=  - 3* ($d-$i) * (-1)**($i)*F($i);
         }
         $total += (F(3*$d+1) - 42*$d) / 4;
         $total +=  -3 * (-1)**$d * F($d)  ;
         $total += -12 * (-1)**$d * F($d-1);
         $total += F(3*$d+4)*3/2;
         $total += -199/4;
         return $total / 30;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           $total +=  - ($d-$i) * 3*(-1)**($i)*F($i);
         }
         $total += (F(3*$d+1) - 2*$d - 1) / 4   - 2*($d-1) - 8*($d-2);
         $total +=   3 * (- (-1)**$d * F($d)   + 1 );
         $total += -12 * (  (-1)**$d * F($d-1) - 1 );
         $total += (F(3*$d+4)*3 - 165)/2;  # sum 3*F(3*$i+2) i=3 to i=d
         return $total / 30;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           # $total += + 3* (-1)**($i+1)*F($i+1);
           $total += ($d-$i) * F(3*$i) - ($d-$i) * 3*(-1)**($i)*F($i);
         }
         $total +=   3 * (- (-1)**$d * F($d)   + 1 );
         $total += -12 * (  (-1)**$d * F($d-1) - 1 );
         $total += (F(3*$d+4)*3 - 165)/2;  # sum 3*F(3*$i+2) i=3 to i=d
         return $total / 30;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           $total +=  - (-1)**$i * 12*F($i) - (-1)**$i * 3*F($i+1) + ($d-$i) * F(3*$i) - ($d-$i) * 3*(-1)**($i)*F($i);
         }
         $total += F(3*$d+4)*3/2 - 165/2;  # sum 3*F(3*$i+2) i=3 to i=d
         return $total / 30;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           $total += 3*F(3*$i+2) - (-1)**$i * (12*F($i) + 3*F($i+1))  +  ($d-$i) * F(3*$i) - ($d-$i) * 3*(-1)**($i)*F($i);
         }
         return $total / 30;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           $total += (3*F(3*$i+2) - (-1)**$i * (12*F($i) + 3*F($i+1))  +  ($d-$i) * (F(3*$i) - 3*(-1)**($i)*F($i)) ) / 30;
         }
         return $total;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           $total += ((F(3*$i+2) - (-1)**$i * (4*F($i) + F($i+1))) / 5) / 2  +  ($d-$i) * (F(3*$i) - 3*(-1)**($i)*F($i)) / 5 / 6;
         }
         return $total;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           $total += F($i)**2 * F($i+2) / 2;
           $total += ($d-$i) * F($i)**3 / 6;
         }
         return $total;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           $total += (3*F($i+2) + ($d-$i)*F($i)) * F($i)**2 / 6;
         }
         return $total;
       },
       sub {
         my ($d) = @_;
         my $total = 0;
         for (my $i = ($d*0)+3; $i <= $d; $i++) {
           $total += (3*F($i+2) + ($d-$i)*F($i)) / 6 * 2 * F($i)**2 / 2;
         }
         return $total;
       },
     ],
     
     #----------------------------------------------------
     [ "hull centroid -- total weighted (numerator)",
       { anum => "grep",
         n_start => 2,
       },
       sub {
         my ($d) = @_;
         my $zero = $d*0;
         my @points = ([$zero+1,$zero+1]);
         foreach my $i (range($d)) {
           my ($p,$q) = hull_vertex($i,$d);
           push @points, [$p,$q];
           unshift @points, [$q,$p];
         }
         my $planar = Math::Geometry::Planar->new;
         $planar->points(\@points);
         my $centroid = $planar->centroid;
         my $area = $planar->area;
         $centroid->[0] == $centroid->[1] or die;
         return $centroid->[0] * $area;
       },
       
       
       sub {
         my ($d) = @_;
         my $coord = 0;
         
         foreach my $i (range($d-1)) {
           my $area = (+   F(2*$i+3)
                       + 2*F(2*$i+2)
                       + 5 * ($d-$i-2) * F($i)
                       - 4 * (-1)**$i
                      ) / 10;
           my $c = (+ ($d-$i+2)*F($i+3)
                    + 2
                   ) / 6;
           my $ca = $area * $c;
           $coord += 2*$ca;
         }
         $coord += (+ F($d) * F($d+3) * F($d+3)
                    - F($d) * F($d+3)
                    - 2* F($d)
                   )/ 6;
         return $coord;
       },
       sub {
         my ($d) = @_;
         my $coord = 0;
         
         foreach my $i (range($d-1)) {
           my $area = (+ F($i+1) * F($i+2)
                       + ($d-$i-2) * F($i)
                       - (-1)**$i
                      ) / 2;
           my $c = (+ ($d-$i+2)*F($i+3)
                    + 2
                   ) / 6;
           my $ca = $area * $c;
           $coord += 2*$ca;
         }
         $coord += (+ F($d) * F($d+3) * F($d+3)
                    - F($d) * F($d+3)
                    - 2* F($d)
                   )/ 6;
         return $coord;
       },
       sub {
         my ($d) = @_;
         my $coord = 0;
         
         foreach my $i (range($d-1)) {
           my $area = (- (-1)**$i
                       + F($i+1) * F($i+2)
                       + ($d-$i) * F($i)
                       - 2*F($i)
                      ) / 2;
           my $c = (+ F($i+5)
                    - F($i+2)
                    + ($d-$i)*F($i+3)
                    + 2
                   ) / 6;
           my $ca = $area * $c;
           $coord += 2*$ca;
         }
         $coord += (+ F($d) * F($d+3) * F($d+3)
                    - F($d) * F($d+3)
                    - 2* F($d)
                   )/ 6;
         return $coord;
       },
       sub {
         my ($d) = @_;
         my $coord = 0;
         
         foreach my $i (range($d-1)) {
           my $area = (-             F($i+1)* F($i+3)
                       - ($d-$i-1) * F($i+1)* F($i+2)
                       +         F($i+2)*F($i+2)
                       + ($d-$i)*F($i+1)*F($i+2)
                       - ($d-$i) *F($i+1)
                       + ($d-$i)*F($i+2)
                       - F($i)
                       - F($i)
                      ) / 2;
           my $c = (+ F($i+5)
                    - F($i+2)
                    + ($d-$i)*F($i+3)
                    + 2
                   ) / 6;
           my $ca = $area * $c;
           $coord += 2*$ca;
         }
         $coord += (+ F($d) * F($d+3) * F($d+3)
                    - F($d) * F($d+3)
                    - 2* F($d)
                   )/ 6;
         return $coord;
       },
       sub {
         my ($d) = @_;
         my $coord = 0;
         
         foreach my $i (range($d-1)) {
           my $area = (- F($i+1)*F($i+3)
                       - F($i+1)* ($d-$i-1)*F($i+2)
                       + F($i+1)
                       + F($i+3)
                       + ($d-$i-1)*F($i+2)
                       - 1
                       +         F($i+2)*F($i+2)
                       + ($d-$i)*F($i+1)*F($i+2)
                       - F($i+2)
                       - (F($i+2) + ($d-$i)  *F($i+1) - 1)
                      ) / 2;
           my $c = (+ F($i+5)
                    - F($i+2)
                    + ($d-$i)*F($i+3)
                    + 2
                   ) / 6;
           my $ca = $area * $c;
           $coord += 2*$ca;
         }
         $coord += (+ F($d) * F($d+3) * F($d+3)
                    - F($d) * F($d+3)
                    - 2* F($d)
                   )/ 6;
         return $coord;
       },
       sub {
         my ($d) = @_;
         my $coord = 0;
         
         foreach my $i (range($d-1)) {
           my $area = (- (F($i+1) - 1)*(F($i+3) + ($d-$i-1)*F($i+2) - 1)
                       + (F($i+2) - 1)*(F($i+2) + ($d-$i)  *F($i+1) - 1)) / 2;
           my $c = (+ F($i+5)
                    - F($i+2)
                    + ($d-$i)*F($i+3)
                    + 2
                   ) / 6;
           my $ca = $area * $c;
           $coord += 2*$ca;
         }
         $coord += (+ F($d) * F($d+3) * F($d+3)
                    - F($d) * F($d+3)
                    - 2* F($d)
                   )/ 6;
         return $coord;
       },
       sub {
         my ($d) = @_;
         my $coord = 0;
         
         foreach my $i (range($d-1)) {
           my $p1 = F($i+1);
           my $q1 = F($i+2) + ($d-$i)*F($i+1);
           my $p2 = F($i+2);
           my $q2 = F($i+3) + ($d-$i-1)*F($i+2);
           my $area = TriangleArea([[1,1], [$p2,$q2], [$p1,$q1] ]);
           my $cp = (1 + $p1 + $p2) / 3;
           my $cq = (1 + $q1 + $q2) / 3;
           my $c = ($cp + $cq)/2;
           my $ca = $area * $c;
           $coord += 2*$ca;
         }
         {
           my $p1 = F($d+1);
           my $q1 = F($d+2);
           my $p2 = F($d+2);
           my $q2 = F($d+1);
           my $area = TriangleArea([[1,1], [$p2,$q2], [$p1,$q1] ]);
           my $cp = (1 + $p1 + $p2) / 3;
           my $cq = (1 + $q1 + $q2) / 3;
           my $c = ($cp + $cq)/2;
           my $ca = $area * $c;
           $coord += $ca;
         }
         return $coord;
       },
       
       
       # {
       #   my $total_by_formula = MyRationals::path_tree_depth_to_hull_area($path,$d);
       #   $weight == MyRationals::path_tree_depth_to_hull_area($path,$d)
       #     or die "d=$d weight=$weight total=$total_by_formula";
       # }
       
       sub {
         my ($d) = @_;
         
         return (
                 F(3*$d+7)
                 - 4 * $d*$d
                 - 22 * $d
                 - 13
                ) / 24;
       },
       sub {
         my ($d) = @_;
         # subtracting side triangles
         
         my $coord = (
                      + 48 * F(3*$d+1)
                      + 40 * F(3*$d)
                      + 17 * F(3*$d+1)
                      - 20 * $d*$d
                      - 110 * $d
                      -65
                     );
         return $coord / 120;
       },
       sub {
         my ($d) = @_;
         # subtracting side triangles
         
         my $coord = (
                      + 12 * F(3*$d+6)
                      - 2 * F(3*$d+5)
                      - 3 * F(3*$d+4)
                      - 8 * F(3*$d)
                      - 12 * F(3*$d+1)
                      - 20 * $d*$d
                      - 110 * $d
                      -65
                     );
         return $coord / 120;
       },
       sub {
         my ($d) = @_;
         # subtracting side triangles
         
         my $coord = (
                      + 12 * F(3*$d+6)
                      - 2 * F(3*$d+5)
                      - 3 * F(3*$d+4)
                      - 60 * F($d)*F($d)*F($d+1)
                      - 40 * F($d)*F($d)*F($d)
                      - 36 * (-1)**$d * F($d+2)
                      + 24 * (-1)**$d * F($d+1)
                      - 20 * $d*$d
                      - 110 * $d
                      -65
                     );
         return $coord / 120;
       },
       sub {
         my ($d) = @_;
         # subtracting side triangles
         
         my $zero = $d*0;
         my $coord = (
                      + 12 * F(3*$d+6)
                      - 2 * F(3*$d+5)
                      - 3 * F(3*$d+4)
                      - 120 * F($d)*F($d)*F($d+4) / 6
                      - 36 * (-1)**$d * F($d+2)
                      + 24 * (-1)**$d * F($d+1)
                      - 20 * $d*$d
                      - 110 * $d
                      -65
                     );
         return $coord / 120;
       },
       sub {
         my ($d) = @_;
         # subtracting side triangles
         
         my $zero = $d*0;
         my $coord = (
                      - 2 * F(3*$d+5)
                      - 3 * F(3*$d+4)
                      + 60  * F($d+2)**3
                      - 120 * F($d)*F($d)*F($d+4) / 6
                      + 24 * (-1)**$d * F($d+1)
                      -20 * $d*$d
                      + 30*$d
                      - 120 * $d*7/6
                      -425
                      + 120 * 3
                     );
         return $coord / 120;
       },
       sub {
         my ($d) = @_;
         # subtracting side triangles
         
         my $zero = $d*0;
         my $coord = 0;
         $coord += F($d+2)**3 / 2;             # square
         $coord -= F($d)*F($d)*F($d+4) / 6;    # top middle
         $coord -= $d*$d*1/6 + $d*7/6 - 3;     # 3 to d+1
         $coord -= ($zero + 11)/2;             # 0,0 to 0,3 and 3,0
         
         $coord -= (2 * F(3*$d+5)
                    + 3 * F(3*$d+4)
                    -24 * (-1)**$d * F($d+1)
                    - 30*$d
                    -235) / 120;
         return $coord;
       },
       
       sub {
         my ($d) = @_;
         # subtracting side triangles
         
         my $zero = $d*0;
         my $coord = 0;
         my $weight = 0;
         
         {
           # square  (1/2)*F(d+2)^3
           my $c = F($d+2)/2;
           my $area = F($d+2)**2;
           $coord += $c*$area;
           $weight += $area;
         }
         
         {
           # top middle corner   (1/6)* F(d+4)*F(d)^2
           my $area = F($d)**2/2;
           my $c = (F($d+1) + F($d+2) + F($d+2)) / 3;
           # $c = F($d+4) / 3;
           $coord -= $c*$area;
           $weight -= $area;
         }
         
         {
           # 3 to d+1 =
           my $area = ($d+1-3)/2;
           $area = $d/2 - 1;
           my $cp = (0+1+1)/3;
           $cp = 2/3;
           my $cq = (3+3+ $d+1)/3;
           $cq = ($d+7)/3;
           my $c = ($cp + $cq)/2;  # (d+7+2)/3/2
           $c = (1/6)*$d + 3/2;
           my $ca2 = $c * 2*$area;
           $ca2 = ((1/6)*$d + 3/2)*($d/2 - 1)*2;
           $ca2 = ((1/6)*$d + 3/2)*($d - 2);
           $ca2 = (1/6)*$d*$d + (3/2 - 1/3)*$d - 3;
           $ca2 = $d*$d*1/6 + $d*7/6 - 3;
           $coord -= $ca2;
           $weight -= 2*$area;
         }
         
         {
           # 0,0 to 0,3 and 3,0
           # .5  weight 1
           # 1   weight 2
           # 1.5 weight 2
           # c = (1/2 + 2*1 + 2*3/2)/5
           #   = 11/10
           my $area = 5;
           my $c = 11/10;
           my $ca = ($zero + 11)/2;
           $coord -= $ca;
           $weight -= $area;
         }
         
         # $coord = Math::BigRat->new($weight);
         # $weight = Math::BigRat->new($weight);
         foreach my $i (3 .. $d) {
           ### $i
           my $area = F($i)**2 / 2;
           
           my $cp = (0 + 0 + F($i)) / 3;
           my $cq = (F($i+1)    # q axis
                     + F($i+2)  # q axis
                     + F($i+1) + ($d-$i+1)*F($i)  # sheared
                    ) / 3;
           my $c = ($cp + $cq)/2;
           $c = (3*F($i+2) + ($d-$i)*F($i)) / 6;
           my $ca2 = $c * 2*$area;
           # $ca2 = (3*F($i+2) + ($d-$i)*F($i)) / 6 * 2 * F($i)**2 / 2;
           # $ca2 = (3*F($i+2) + ($d-$i)*F($i)) * F($i)**2 / 6;
           # $ca2 = (1/2)* F($i)**2 * F($i+2)  +  (1/6)*($d-$i) * F($i)**3;
           
           $coord -= $ca2;
           $weight -= 2*$area;
         }
         return $coord;
       },
       
     ],
     
     #----------------------------------------------------
     [ "F(n)*F(n+1)",
       { anum => "A001654" },  # golden rectangle
       \&pairF,
       
       sub {
         my ($n) = @_;
         return (+ F(2*$n+2)
                 + F(2*$n)
                 - (-1)**$n) / 5;
       },
       sub {
         my ($n) = @_;
         return (F(2*$n+1+1) + F(2*$n+1-1) - (-1)**$n)/5;
       },
       sub {
         my ($n) = @_;
         return (Lucas(2*$n+1) -(-1)**$n)/5;
       },
     ],
     
     
     #----------------------------------------------------
     [ "middle hull triangle",
       { anum => "grep",
       },
       sub {
         my ($d) = @_;
         my $p1 = F($d+1);
         my $q1 = F($d+2);
         my $p2 = F($d+2);
         my $q2 = F($d+1);
         my $area = TriangleArea([[1,1], [$p2,$q2], [$p1,$q1] ]);
         my $cp = (1 + $p1 + $p2) / 3;
         my $cq = (1 + $q1 + $q2) / 3;
         my $c = ($cp + $cq)/2;
         my $ca = $area * $c;
         return $ca;
       },
       
       sub {
         my ($d) = @_;
         return (+ F($d) * F($d+3) * F($d+3)
                 - F($d) * F($d+3)
                 - 2* F($d)
                )/ 6;
       },
       sub {
         my ($d) = @_;
         return (F($d+3) - 2) * F($d) * (F($d+3) + 1) / 6;
       },
       sub {
         my ($d) = @_;
         return (F($d+2) - 1 + F($d+1) - 1) * (F($d+2) - 1 - F($d+1) + 1) * (F($d+3) + 1) / 6;
       },
       sub {
         my ($d) = @_;
         return ((F($d+2) - 1)**2 - (F($d+1) - 1)**2) * (F($d+3) + 1) / 6;
       },
       sub {
         my ($d) = @_;
         my $p1 = F($d+1);
         my $q1 = F($d+2);
         my $p2 = F($d+2);
         my $q2 = F($d+1);
         my $area = ((F($d+2) - 1)**2 - (F($d+1) - 1)**2) / 2;
         my $c = (F($d+3) + 1) / 3;
         my $ca = $area * $c;
         return $ca;
       },
       
     ],
     
     #----------------------------------------------------
     [ "sum i=1 to n of (n-i) * (-1)^i * F(i)",
       { anum => "grep",
         n_start => 1,
       },
       \&sum_nsubi_altF,
       
       # eg n=6
       # -F(1) +F(2) -F(3) + F(4) -F(5)
       # -F(1) +F(2) -F(3) + F(4)
       # -F(1) +F(2) -F(3)
       # -F(1) +F(2)
       # -F(1)
       # sum i=1 to n-1 of (sum (-1)^i*F(i))
       # sum_altF = (-1)**$n * F($n-1) - 1
       sub {
         my ($n) = @_;
         return - (-1)**$n * F($n-3) - $n + 2;
       },
       sub {
         my ($n) = @_;
         return - ((-1)**$n * F($n-1) - 1)
           + (-1)**$n * F($n)
             - (-1)**$n * F($n-1)
               - $n + 1
                 ;
       },
       sub {
         my ($n) = @_;
         return - sum(0, map {(-1)**($_) * F($_)} range($n))
           + (-1)**$n * F($n)
             - $n + 1
               - (-1)**$n * F($n-1);
       },
       sub {
         my ($n) = @_;
         return - sum(0, map {(-1)**($_) * F($_)} range($n-1))
           - $n + 1 - (-1)**$n * F($n-1);
       },
       sub {
         my ($n) = @_;
         return - sum(0, map {(-1)**($_-1) * F($_-1)} range($n)) - $n + 1 - (-1)**$n * F($n-1);
       },
       sub {
         my ($n) = @_;
         return - sum(0, map {(-1)**($_-1) * F($_-1)} range($n-1)) - $n + 1;
       },
       sub {
         my ($n) = @_;
         return sum(0, map {(-1)**$_ * F($_-1) - 1} range($n-1));
       },
       sub {
         my ($n) = @_;
         return sum(0, map {sum_altF($_)} range($n-1));
       },
     ],

     #----------------------------------------------------
     [ "sum i=1 to n of (n-i)*F(3i)",
       { anum => "grep",
         n_start => 1,
       },
       \&sum_nsubi_F3i,

       # eg. n=5
       # F(3) + F(6) + F(9) + F(12)
       # F(3) + F(6) + F(9)
       # F(3) + F(6)
       # F(3)
       # with sum F(3n) = (F(3n+2) - 1)/2
       # is sum i=1 to n-1 of (F(3n+2) - 1)/2
       sub {
         my ($n) = @_;
         return sum(0, map {(F(3*$_+2) - 1)/2} range($n-1));
       },
       sub {
         my ($n) = @_;
         return (sum(0, map {F(3*$_+2)} range($n-1)) - $n + 1) / 2;
       },
       # with sum F(3n+2) = (F(3*$n+4) - 3)/2;   # for n>=1
       sub {
         my ($n) = @_;
         return (F(3*$n+1) - 2*$n - 1) / 4;
       },
     ],


     #----------------------------------------------------
     [ "sum i=1 to n of (-1)^i * F(i)",
       { anum => "A119282" },
       \&sum_altF,
       sub {
         my ($n) = @_;
         return (-1)**$n * F($n-1) - 1;
       },
     ],

     #----------------------------------------------------
     [ "sum i=1 to n of F(3i+2)",
       { anum => "grep" },
       \&sum_F3iplus2,

       sub {
         my ($n) = @_;
         return (F(3*$n+4) - 3)/2;
       },
     ],

     #----------------------------------------------------
     [ "sum i=1 to n of F(3i+1)",
       { anum => "grep" },
       \&sum_F3iplus1,

       sub {
         my ($n) = @_;
         return F(3*$n+3)/2 - 1;
       },
     ],

     #----------------------------------------------------
     [ "sum i=1 to n of F(3i)",
       { anum => "A099919" },
       \&sum_F3i,

       # A099919 = F(3) + F(6) + ...
       #  = 2*A049652(n)         A049652=(F(3n+2)-1)/4
       #  = 2*(F(3n+2)-1)/4
       #  = (F(3n+2) - 1)/2
       sub {
         my ($n) = @_;
         return (F(3*$n+2) - 1)/2;
       },
     ],

     #----------------------------------------------------
     [ "full square centroid weighted",
       { anum => "A056570 cubes / 2" },
       sub {
         my ($d) = @_;
         my $c = F($d+2)/2;
         my $area = F($d+2)**2;
         return $c*$area;
       },

       # square  (1/2)*F(d+2)^3
       sub {
         my ($d) = @_;
         return F($d+2)**3 / 2;
       },
     ],

     #----------------------------------------------------
     [ "hull centroid top middle corner",
       { anum => "grep" },
       sub {
         my ($d) = @_;
         my $area = F($d)**2/2;
         my $c = (F($d+1) + F($d+2) + F($d+2)) / 3;
         return $c*$area;
       },

       # top middle corner   (1/6)* F(d+4)*F(d)^2
       sub {
         my ($d) = @_;
         return F($d)*F($d)*F($d+4) / 6;
       },

       sub {
         my ($d) = @_;
         return F($d)*F($d)*(2*F($d) + 3*F($d+1)) / 6;
       },
       sub {
         my ($d) = @_;
         return F($d)*F($d)*F($d) /3 + F($d)*F($d)*F($d+1) /2;
       },
     ],

     #----------------------------------------------------
     [ "F(n)^3",
       { anum => "A056570" },
       \&Fcubed,

       # F(n)^3 = (F(3n) - 3(-1)^n*F(n)) / 5
       sub { my ($n) = @_;
             return (F(3*$n) - 3 * (-1)**$n * F($n)) / 5;
           },
     ],

     #----------------------------------------------------
     [ "F(n)*F(n)*F(n+1)",
       { anum => "A066258" },
       sub {
         my ($n) = @_;
         return F($n)*F($n)*F($n+1);
       },

       # cf A066258(n) = F(n)^2*F(n+1)
       #               = (1/5)*(F(3n+1) - (-1)^n*F(n+2))
       sub {
         my ($n) = @_;
         return (F(3*$n+1) - (-1)**$n * F($n+2)) / 5;
       },
     ],

     #----------------------------------------------------
     [ "sum i=1 to n of F(i)*F(i+1)*F(i+1)",
       { anum => "A215038" },
       \&sum_Fi_Fi1_Fi1,

       # A215038 Partial sums of A066259: a(n) = sum(F(k+1)^2*F(k),k=0..n)
       #  = (F(n+1)^2 * F(n+2)  - (-1)^n*(F(n) + (-1)^n)/2
       #  = (F(n+1)^2 * F(n+2)  - (-1)^n*F(n) - (-1)^n*(-1)^n )/2
       #  = (F(n+1)^2 * F(n+2)  - (-1)^n*F(n) - 1 )/2
       #
       # cf A066258(n) = F(n)^2*F(n+1)
       #               = (1/5)*(F(3n+1) - (-1)^n*F(n+2))
       sub {
         my ($n) = @_;
         return ( F($n+1)**2 * F($n+2)  - (-1)**$n * F($n) - 1 )/2;
       },
     ],

     #----------------------------------------------------
     [ "sum i=1 to n of F(i)*F(i)*F(i+1)",
       { anum => "A001655" },
       \&sum_Fi_Fi_Fi1,

       # sum i=1 to n of F(i)*F(i)*F(i+1) = F(n)*F(n+1)*F(n+2)/2
       sub {
         my ($n) = @_;
         return F($n)*F($n+1)*F($n+2)/2;
       },
     ],

     #----------------------------------------------------
     [ "sum i=1 to n of F(i)^3",
       { anum => "A005968" },
       \&sum_Fcubed,

       # sum i=1 to n of F(n)^3 = (F(3n+2) - 6*(-1)^(n)*F(n-1) + 5) / 10
       sub {
         my ($n) = @_;
         return (F(3*$n+2) - 6*(-1)**$n * F($n-1) + 5) / 10;
       },
     ],

     #----------------------------------------------------
     [
      "sum i=1 to n of i*F(i)",
      { anum => "A014286" },
      sub { my ($n) = @_;
            return sum(0, map { $_*F($_) } range($n));
          },
      sub { my ($n) = @_;
            return $n*F($n+2) - F($n+3) + 2;
          },
     ],

     #----------------------------------------------------

    );

my $count = 0;
foreach my $identity (@identities) {
  my ($name, $options, $want_func, @funcs) = @$identity;
  my @want;
  my $n_start = $options->{'n_start'} // 0;
  my $n_end = $options->{'n_end'} // $n_start + 20;
  foreach my $func_num (0 .. $#funcs) {
    my $func = $funcs[$func_num];
    my $heading = "$name;  func $func_num\n";
    for (my $n = Math::BigRat->new($n_start); $n < $n_end; $n++) {
      my $want = ($want[$n] //= $want_func->($n));
      my $got = $func->($n);
      if ($got != $want) {
        print "${heading}n=$n want=$want got=$got\n";
        $heading = '';
      }
      $count++;
    }
  }

  if ($options->{'anum'} eq 'grep') {
    shift @want; shift @want; shift @want;
    require Math::OEIS::Grep;
    Math::OEIS::Grep->search(array => \@want,
                             name => $name);
  }
}
print "count $count\n";
exit 0;



sub range {
  my ($n) = @_;
  my $zero = $n*0;
  return map {$zero + $_} 1 .. $n;
}

sub pairF {
  my ($n) = @_;
  return F($n)*F($n+1);
}
sub tripF {
  my ($n) = @_;
  return F($n)*F($n+1)*F($n+2);
}
sub sum_nsubi_altF {
  my ($n) = @_;
  return sum(0, map {($n-$_) * (-1)**$_ * F($_)} range($n));
}
sub sum_altF {
  my ($n) = @_;
  return sum(0, map {(-1)**$_ * F($_)} range($n));
}
sub sum_twoF {
  my ($n) = @_;
  my $sum = 0;
  foreach my $i (1 .. $n) {
    $sum += F(2*$i-1);
  }
  return $sum;
}
  sub sum_nsubiF {
    my ($n) = @_;
    my $sum = 0;
    foreach my $i (1 .. $n) {
      $sum += ($n-$i)*F($i);
    }
    return $sum;
  }
  sub sum_iF {
    my ($n) = @_;
    my $sum = 0;
    foreach my $i (1 .. $n) {
      $sum += $i*F($i);
    }
    return $sum;
  }
  sub sum_2iF {
    my ($n) = @_;
    # 2*$n*F(2*$n+1) - 2*F(2*$n);     # A197649
    my $sum = 0;
    foreach my $i (1 .. $n) {
      $sum += 2*$i*F(2*$i);
    }
    return $sum;
  }
  sub sum_2im1F {
    my ($n) = @_;
    # (2*$n+1)*F(2*$n) - 2*F(2*$n+1) + 2;    # not in OEIS
    my $sum = 0;
    foreach my $i (1 .. $n) {
      $sum += (2*$i-1)*F(2*$i-1);
    }
    return $sum;
  }

sub sum_nsubi_F3i {
  my ($n) = @_;
  return sum(0, map {($n-$_)*F(3*$_)} range($n));
}
sub sum_F3i {
  my ($n) = @_;
  return sum(0, map {F(3*$_)} range($n));
}
sub sum_F3iplus1 {
  my ($n) = @_;
  return sum(0, map {F(3*$_+1)} range($n));
}
sub sum_F3iplus2 {
  my ($n) = @_;
  return sum(0, map {F(3*$_+2)} range($n));
}

sub Fcubed {
  my ($n) = @_;
  return F($n)**3;
}
sub sum_Fcubed {
  my ($n) = @_;
  return sum(0, map {F($_)**3} range($n));
}
sub sum_Fi_Fi_Fi1 {
  my ($n) = @_;
  return sum(0, map {F($_)*F($_)*F($_+1)} range($n));
}
sub sum_Fi_Fi1_Fi1 {
  my ($n) = @_;
  return sum(0, map {F($_)*F($_+1)*F($_+1)} range($n));
}

sub sum_F3iminus1 {
  my ($n) = @_;
  # A049651 a(n)=(F(3n+1) - 1)/2
  my $sum = 0;
  foreach my $i (1 .. $n) {
    $sum += F(3*$i-1);
  }
  return $sum;
}

sub hull_vertex {
  my ($i,$d) = @_;
  return (F($i+1),
          F($i+2) + ($d-$i)*F($i+1));
}

__END__

{
  # sum i=1 to i=n of i*F(i)  = A014286
  # F(1) + F(2) + F(3) + F(4) = F(6)-1 - (F(2)-1)
  #        F(2) + F(3) + F(4) = F(6)-1 - (F(3)-1)
  #               F(3) + F(4) = F(6)-1 - (F(4)-1)
  #                      F(4) = F(6)-1 - (F(5)-1)
  # = n*(F(n+2)-1) + n - (F(n+3)-2)
  # = n*F(n+2) - F(n+3) + 2
  #
  # sum i=1 to i=n of (n-i)*F(i) = A001924
  # 3*F(1) + 2*F(2) + 1*F(3) + 0*F(4) = F(6)-1
  # F(1) + F(2) + F(3)  = F(5)-1
  # F(1) + F(2)         = F(4)-1
  # F(1)                = F(3)-1
  # = F(n+3)-1 - (n-1) - F(2) - F(1)
  # = F(n+3) - n - 2
  #
  # sum i=1 to i=n of F(2i)  = F(2n+1)-1 = A027941
  # = F(2) + F(4) + F(6)
  #
  # sum i=1 to i=n of F(2i+1) = F(2n+2)-1 = A035508
  # = F(3) + F(5) + F(7)
  #
  # sum i=1 to i=n of F(2i-1) = F(2n) = A001906
  # = F(1) + F(3) + F(5) + F(7)
  #
  # sum i=1 to i=n of (-1)^i*F(i)  = A119282
  #        - F(1) + F(2) - F(3) + F(4) - F(5) + F(6)
  # = F(0)        + F(2)        + F(4)
  #        - F(1) + F(2) - F(3) + F(4) - F(5) + F(6) - F(7)
  #       -2*F(1)        - F(3)        - F(5)
  # = (-1)^n*F(n-1) - 1
  #
  # sum i=1 to i=n of i*(-1)^i*F(i)  =
  # - F(1) + F(2) - F(3) + F(4) = F(6)-1 - (F(2)-1)
  #          F(2) - F(3) + F(4) = F(6)-1 - (F(3)-1)
  #                 F(3) + F(4) = F(6)-1 - (F(4)-1)
  #                        F(4) = F(6)-1 - (F(5)-1)
  # = n*(F(n+2)-1) + n - (F(n+3)-2)
  # = n*F(n+2) - F(n+3) + 2
  #
  #
  # 2, 18, 180, 1200, 7800, 45864, 259896, 1413720, 7489350, 38768400,
  # 270, 1875, 10752, 61516, 333396, 1768680, 9150625,
  #
  my @values;
  foreach my $n (1 .. 20) {
    my ($s,$f);
    # my $s = sum_nsubiF($n);
    # my $f = F($n+3) - $n - 2;
    # my $s = sum_ialtF($n);
    # my $f = $n*F($n+2) - F($n+3) + 2;
    # my $s = sum_twoF($n);
    # my $f = $n*F($n+2) - F($n+3) + 2;
    # my $s = sum_2iF($n);
    # my $f = 2*$n*F(2*$n+1) - 2*F(2*$n);
    # my $s = sum_2im1F($n);
    # my $f = (2*$n+1)*F(2*$n) - 2*F(2*$n+1) + 2;
    $s = sum_F3iminus2($n);
    $f = F(3*$n)/2;
    $s = sum_nsubi_tripF($n);
    $f = F(3*$n+2);
    $s = sum_tripF($n);
    $f = (F($n+1)*F($n+2)**2 - (-1)**$n * F($n) - 1 )/2;
    $f =  ( (1/5) * F(3*$n+5) - (4/5)*(-1)**$n * F($n) - 1)/2;
    $f =  (1/10) * F(3*$n+5) - (2/5)*(-1)**$n * F($n) - 1/2;
    print "$s $f\n";
    # $s == $f or die;
    push @values, $s;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;

  sub sum_F3iminus2 {
    my ($n) = @_;
    # A001076 Denominators of continued fraction convergents to sqrt(5).
    #         = F(3n)/2         # F(3n) is even
    my $sum = 0;
    foreach my $i (1 .. $n) {
      $sum += F(3*$i-2);
    }
    return $sum;
  }
  sub sum_nsubi_tripF {
    my ($n) = @_;
    # (n-i)*F(i)*F(i+1)*F(i+2)
    # = 2*F(1)*F(2)*F(3) + 1*F(2)*F(3)*F(4) + 0*F(3)*F(4)*F(5)
    # = sum i=1 to n-1 of sum_tripF(i)
    # = sum of (1/10) * F(3*$n+5) - (2/5)*(-1)**$n * F($n) - 1/2
    my $sum = 0;
    foreach my $i (1 .. $n) {
      $sum += ($n - $i) * F($i) * F($i+1) * F($i+2);
    }
    return $sum;
  }
  sub sum_tripF {
    my ($n) = @_;
    # A215037 a(n) = sum(fibonomial(k+3,3), k=0..n), n>=0.
    # a(n) = ( F(n+2)*F(n+3)^2 + (-1)^n*(F(n+1) + (-1)^(n+1)) )/4
    #      = ( (1/5) F(3(n+2)+2) - (1/5)*(-1)^(n+2)F(n+2-1)
    #              + (-1)^n*(F(n+1) + (-1)^(n+1)) )/4
    #      = ( (1/5) F(3n+8) + (4/5)*(-1)^n * F(n+1) - (-1)^n )/4
    # $f =  ( (1/5) * F(3*$n+5) - (4/5)*(-1)**$n * F($n) - 1)/2;
    # $f =  (1/10) * F(3*$n+5) - (2/5)*(-1)**$n * F($n) - 1/2;
    #
    # cf
    # A066259 Fibonacci(n)*Fibonacci(n+1)^2
    # a(n) = (1/5) {F(3n+2) - (-1)^nF(n-1) }
    #
    my $sum = 0;
    foreach my $i (1 .. $n) {
      $sum += tripF($i);
    }
    return $sum;
  }

}
