#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use File::Slurp;
use File::stat;
use List::Util 'max','min';
use Image::Base::GD;
use Math::PlanePath::RationalsTree;
use POSIX 'ceil';
use IPC::Run;

# uncomment this to run the ### lines
# use Smart::Comments;


my $depth = 3;
my $xscale = 3;
my $yscale = 3;

my $version = 0;
foreach my $arg (@ARGV) {
  if ($arg =~ /^VERSION=(.*)/) {
    $version = $1;
  }
}

my $program_filename = File::Spec->catfile($FindBin::Bin,
                                           $FindBin::Script);
my $filename = File::Spec->catfile($FindBin::Bin,
                                   File::Spec->updir,
                                   'favicon.png');

my $copyright = File::Slurp::read_file($program_filename);
$copyright =~ /^# (Copyright.*)/m or die;
$copyright = $1;

my $path = Math::PlanePath::RationalsTree->new;
my ($n_lo, $n_hi) = $path->tree_depth_to_n_range($depth);
print "N= $n_lo to $n_hi\n";
my (@x,@y);
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  push @x, $x;
  push @y, $y;
}
### @x
my $x_min = min(@x);
my $y_min = min(@y);
my $x_max = max(@x);
my $y_max = max(@y);
### $x_min
### $y_min
### $x_max
### $y_max

my $size = 16;
my $width  = $x_max - $x_min;
my $height = $y_max - $y_min;

my $x_base = int(($size - $width*$xscale)/2) - $x_min*$xscale;
my $y_base = ceil(($size - $height*$yscale)/2) - $y_min*$yscale;
### $width
### $height
### $x_base
### $y_base
print "X $x_min to $x_max   Y $y_min to $y_max  width=$width height=$height  X base $x_base Y base $y_base\n";

my $image = Image::Base::GD->new (-height => $size,
                                  -width  => $size);
$image->rectangle(0,0, $size-1,$size-1, 'black');
foreach my $i (0 .. $#x-1) {
  my $x1 = $x[$i];
  my $y1 = $y[$i];
  my $x2 = $x[$i+1];
  my $y2 = $y[$i+1];
  $x1 *= $xscale;
  $y1 *= $yscale;
  $x2 *= $xscale;
  $y2 *= $yscale;
  $x1 += $x_base;
  $y1 += $y_base;
  $x2 += $x_base;
  $y2 += $y_base;
  $y1 = $size-1 - $y1;
  $y2 = $size-1 - $y2;
  ### line: "$x1,$y1 to $x2,$y2"
  $image->line($x1,$y1, $x2,$y2, 'white');
}

foreach my $depth (0 .. $depth-1) {
  my ($n_lo, $n_hi) = $path->tree_depth_to_n_range($depth);
  my (@x,@y);
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    push @x, $x;
    push @y, $y;
  }
  foreach my $i (0 .. $#x-1) {
    my $x1 = $x[$i];
    my $y1 = $y[$i];
    my $x2 = $x[$i+1];
    my $y2 = $y[$i+1];
    $x1 *= $xscale;
    $y1 *= $yscale;
    $x2 *= $xscale;
    $y2 *= $yscale;
    $x1 += $x_base;
    $y1 += $y_base;
    $x2 += $x_base;
    $y2 += $y_base;
    $y1 = $size-1 - $y1;
    $y2 = $size-1 - $y2;
    ### line: "$x1,$y1 to $x2,$y2"
    $image->line($x1,$y1, $x2,$y2, 'white');
  }
}

$image->save($filename);

# PNG spec 11.3.4.2 suggests RFC822 (or rather RFC1123) for CreationTime
use constant STRFTIME_FORMAT_RFC822 => '%a, %d %b %Y %H:%M:%S %z';
my $stat = File::stat::stat($program_filename)
  or die "Oops, cannot stat $program_filename: $!";
my $creation_time = POSIX::strftime(STRFTIME_FORMAT_RFC822,
                                    localtime($stat->mtime));

pngtextadd($filename, 'Author',    'Kevin Ryde');
pngtextadd($filename, 'Generator', "tools/$FindBin::Script");
pngtextadd($filename, 'Title',     'Trees of Rationals - icon');
pngtextadd($filename, 'Creation Time', $creation_time);
pngtextadd($filename, 'Copyright', $copyright);
pngtextadd($filename, 'Homepage',  'http://user42.tuxfamily.org/rationals/index.html');
pngtextadd($filename, 'Version',   $version);

print "uncompressed ",-s $filename,"\n";
IPC::Run::run(['optipng','-quiet','-o5',$filename]);
print "compressed   ",-s $filename,"\n";

if (($ARGV[0]||0) eq 'show') {
  IPC::Run::run(['xzgv',$filename]);
}

exit 0;

# add text to the png image in $filename
sub pngtextadd {
  my ($filename, $keyword, $value) = @_;
  system('pngtextadd', "--keyword=$keyword", "--text=$value", $filename) == 0
    or die "system(pngtextadd)";
}
