#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture showing a minimum area rectangle around the SB tree
# points, being all points rows 0 to $depth.

use 5.010;
use strict;
use List::Util 'max','min';
use Math::Geometry::Planar;
use Math::NumSeq::Fibonacci;
use Math::PlanePath::RationalsTree;

use FindBin;
use lib $FindBin::Bin;
use MyRationals 'F','planar_isboundary';
use MyPrintwrap ':all';


my $depth = 5;

print "  % generated by tools/$FindBin::Script\n";
print "  % depth d=$depth\n";

my $path = Math::PlanePath::RationalsTree->new(tree_type => 'SB');

my $points;
my $x_max = 0;
my $y_max = 0;
foreach my $n ($path->n_start .. $path->tree_depth_to_n_end($depth)) {
  my ($x,$y) = $path->n_to_xy($n);
  push @$points, [$x,$y];
  $x_max = max($x_max, $x);
  $y_max = max($x_max, $y);
}

my $planar = Math::Geometry::Planar->new;
$planar->points($points);
if (@$points > 3) { $planar = $planar->convexhull2; }
$points = $planar->points;
my $i_mid = $#$points/2;
### $points

{
  $Printwrap = 2;
  printwrap("\\draw[line join=round]\n");
  $Printwrap += 2;
  my $join = '';
  foreach my $i (0 .. $#$points) {
    my $p = $points->[$i];
    my ($x,$y) = @$p;
    printwrap("$join($x,$y)");
    $join = '--';
  }
  printwrap("--cycle;\n");
  printwrap_newline();
}

my ($corner_west, $corner_north, $corner_south, $corner_east);
my ($x1,$y1, $x2,$y2);
foreach my $i (0 .. $#$points) {
  my $p1 = $points->[$i-1];
  my $p2 = $points->[$i];
  ($x1,$y1) = @$p1;
  ($x2,$y2) = @$p2;
  if ($x1 > $x2) {
    ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1);
  }
  my $dx = $x2 - $x1;
  my $dy = $y2 - $y1;
  if ($dx == 1 && $dy == 2) {
    print "  % side $x1,$y1 to $x2,$y2\n";
    $corner_west = PerpendicularFoot([$p1,$p2,
                                      [1,1]]);
    $corner_north = PerpendicularFoot([$p1,$p2,
                                       [F($depth+1),F($depth+2)]]);

    $corner_south = PerpendicularFoot([$corner_west,[1,1],
                                       [F($depth+2),F($depth)]]);
    $corner_east = PerpendicularFoot([$corner_north,
                                      [F($depth+1),F($depth+2)],
                                      [F($depth+2),F($depth)]]);
    last;
  }
}

my ($north_x,$north_y) = @$corner_north;
my ($south_x,$south_y) = @$corner_south;
my ($east_x,$east_y) = @$corner_east;
my ($west_x,$west_y) = @$corner_west;

printf "  \\coordinate (p-west)  at (%.3f,%.3f);\n", @$corner_west;
printf "  \\coordinate (p-north) at (%.3f,%.3f);\n", @$corner_north;
printf "  \\coordinate (p-east)  at (%.3f,%.3f);\n", @$corner_east;
printf "  \\coordinate (p-south) at (%.3f,%.3f);\n", @$corner_south;
printf "  \\draw (p-west)--(p-north)--(p-east)--(p-south)--cycle;\n";

# {
#   # width line
#   my $dx = $west_x - $north_x;
#   my $dy = $west_y - $north_y;
#   my $f = .25;
#   printf "  \\coordinate (w-left) at (%.3f,%.3f);\n",
#     $west_x + $dx*$f, $west_y + $dy*$f;
#   printf "  \\coordinate (w-right) at (%.3f,%.3f);\n",
#     $south_x + $dx*$f, $south_y + $dy*$f;
# }
# {
#   # height line
#   my $dx = $south_x - $west_x;
#   my $dy = $south_y - $west_y;
#   my $f = .45;
#   printf "  \\coordinate (h-lower) at (%.3f,%.3f);\n",
#     $south_x + $dx*$f, $south_y + $dy*$f;
#   printf "  \\coordinate (h-upper) at (%.3f,%.3f);\n",
#     $east_x + $dx*$f, $east_y + $dy*$f;
# }

{
  my $count_big = 0;
  my $rectpoly = Math::Geometry::Planar->new;
  $rectpoly->points([$corner_west,$corner_north,$corner_east,$corner_south]);
  $Printwrap = 2;
  foreach my $i (0 .. $#$points) {
    my $p = $points->[$i];
    my $style = '[small dot]';
    my ($x,$y) = @$p;
    if (planar_isboundary($rectpoly, $p)
       && $x != $x1) {
      $count_big++;
      $style = '[big dot]';
    }
    printwrap(" \\fill$style ($x,$y) circle;");
  }
  printwrap_newline();
  printwrap(" % $count_big points big dots\n");
}

print "  % end generated\n";
exit 0;
